#!/usr/bin/perl
#
# LCDproc client for distributed.net client 1.1
# Copyright (C) 2000  Matthew Dainty (madmatt@bits.bris.ac.uk)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: dnetc-client.pl,v 1.3 2000/07/15 03:08:41 matt Exp $

use IO::Socket;
use Fcntl;
use File::Tail 0.91;

# Change this for your setup
my $filename = "/var/log/dnetc/client";

# If a Mkey is 1000 kkeys, and a kkey is 1000 keys, change the number below.
my $multiple = 1024;

# This is the list of projects you want to monitor. Currently you can
# choose from:
#
# o RC5
# o OGR
# o DES
# o CSC
#
# The last two are purely for completion as they are no longer running.
# OGR will probably commence again at some point <shrug>
#
# Example, to monitor RC5 and OGR, change the line below to:
#
# my @projects = ( "RC5", "OGR" );
#
# NOTE: Make sure you get the case right!
my @projects = ( "RC5", "OGR" );

my %stats;
my $speed = "";
my $currproj;

# Quickly rip through the client log file and get the latest stats for all
# the projects
if( -f $filename ) {
	open IN, $filename or die "Cannot open distributed.net client log file\n";
	while( <IN> ) {
		carve( $_ );
	}
	close IN;
} else {
	die "Cannot open distributed.net client log file\n";
}

# We want to now tail this log file for any updates
push( @files, File::Tail->new( name=>$filename, debug=>0 ) );

# Connect to the LCDproc server
$remote = IO::Socket::INET->new(
		Proto     => "tcp",
		PeerAddr  => "localhost",
		PeerPort  => "13666",
	)
	|| die "Cannot connect to LCDproc port\n";

# Make sure our messages get there right away
$remote->autoflush(1);

# Give the server a little time to realise we're here
sleep 1;

# Connect to the LCDproc server (easy-peasy)
print $remote "hello\n";
my $lcdconnect = <$remote>;

# This contains the various bits of info about the LCD screen we are
# displaying on
#
# FIXME: Maybe add code so that this will work on screens other than
# 20x4 in size. 20x2, etc.
# print $lcdconnect;

# Turn off blocking mode
fcntl($remote, F_SETFL, O_NONBLOCK);

# Construct a new LCDproc screen for each project we want to display
print $remote "client_set name {distributed.net clients}\n";
foreach $project ( @projects ) {

	# Add a screen
	print $remote "screen_add $project\n";
	print $remote "screen_set $project priority 128 name {distributed.net $project}\n";

	# Add the title for the screen
	print $remote "widget_add $project $project-title title\n";
	print $remote "widget_set $project $project-title {distributed.net $project}\n";

	# Add the various widgets
	print $remote "widget_add $project $project-cg0 vbar\n";
	print $remote "widget_add $project $project-cg1 vbar\n";
	print $remote "widget_add $project $project-cg2 vbar\n";
	print $remote "widget_add $project $project-cg3 vbar\n";
	print $remote "widget_add $project $project-cg4 vbar\n";
	print $remote "widget_add $project $project-lt scroller\n";
	print $remote "widget_add $project $project-at scroller\n";
	print $remote "widget_add $project $project-bt scroller\n";
}

# Set the widgets up with the initial values
update();

# Loop for the rest of eternity
while( 1 ) {

	# Tail the log file and timeout after 30 seconds
	$nfound = File::Tail::select( undef, undef, undef, 30, @files );
	unless( $nfound ) {

		# We had a timeout, therefore we might like to do something
		# interesting here...
	}

	# Get any new messages from the LCDproc server, (we don't do
	# anything with 'em yet)
	while( defined( <$remote> ) ) {
		
		# Do something with the input, eg. trap for closing...
	}

	# If the log file has some new input to be read
	foreach( @files ) {
		if( ! $_->predict ) {
			carve( $_->read );
		}
	}

	# Update the LCDproc widgets
	update();
	sleep 1;
}

# This is here for syntactic completeness, it never runs!
close( $remote )            || die "close: $!";
exit;

# Little subroutine to split a matching log entry and dump the various
# bits of info into the hash ready for displaying elsewhere
sub carve {
	foreach( @_ ) {
		if( /(Completed) (\S+) / ) {
			$speed = $1;
			$currproj = $2;
		} elsif( /(Summary): [0-9]+ (\S+) / ) {
			$speed = $1;
			$currproj = $2;
		}
		if( /([0-9]+) (\S+) \S+ \(([0-9]+) \S+ \S+\) (remain|remains)/ ) {
			$stats{"pleft-".$2} = $1;
			$stats{"uleft-".$2} = $3;
		} elsif( /([0-9]+) (\S+) \S+ \(([0-9]+) \S+ \S+\) (are|is)/ ) {
			$stats{"pdone-".$2} = $1;
			$stats{"udone-".$2} = $3;
		}
		if( /Retrieved ([0-9]+) \S+ \S+ \(([0-9]+)/ ) {
			$stats{"uleft-".$currproj} += $1;
			$stats{"pleft-".$currproj} += $2;
		} elsif( /Sent ([0-9]+) \S+ \(([0-9]+)/ ) {
			$stats{"pdone-".$currproj} -= $1;
			$stats{"udone-".$currproj} -= $2;
		}
		if( /- \[(.*)\]/ ) {
			if( $speed eq "Completed" ) {
				if( $1 =~ /(\S+)/ ) {

					# Get rid of the comma separator, Perl
					# doesn't like it with regards to the
					# the number format
					$tmp = join( '', split( /,/, $1 ) );

					# The OGR project measures the work
					# unit rate in nodes rather than keys
					# so this hack is here purely to
					# display the correct units for that
					# project, else the default is used
					if( $currproj eq "OGR" ) {
						$measurement = "nodes";
					} else {
						$measurement = "keys";
					}

					# Get the latest completed value into
					# sensible units instead of a huuuuuge
					# keys/s value :)
					if( $tmp > ( $multiple * $multiple ) ) {
						$stats{"rate-".$currproj} = sprintf "%d.2 M%s/s", $tmp / ( $multiple * $multiple ), $measurement;
					} elsif( $tmp > $multiple ) {
						$stats{"rate-".$currproj} = sprintf "%d.2 k%s/s", $tmp / $multiple, $measurement;
					} else {
						$stats{"rate-".$currproj} = sprintf "%d.2 %s/s", $tmp, $measurement;
					}

					# Store the latest 5 rates for
					# displaying. I think this is a messy
					# way of storing 'em, (in a space
					# separated string), as I could use an
					# array or list, but I don't know how
					# to store those in the hash. So if
					# anyone knows, I'd love to hear from
					# you
					$stats{"rates-".$currproj} = $stats{"rates-".$currproj}." ".$tmp;
					@tmp = split( ' ', $stats{"rates-".$currproj} );
					if( $#tmp > 4 ) {
						$stats{"rates-".$currproj} = $tmp[1]." ".$tmp[2]." ".$tmp[3]." ".$tmp[4]." ".$tmp[5];
					}
				}
			} elsif( $speed eq "Summary" ) {

				# I think the distributed.net client
				# automagically uses the best units for the
				# summary value, and also only displays to
				# two decimal places, so I won't bother
				# altering that value
				$stats{"total-".$currproj} = $1;
			}
		}
	}
}

# This subroutine sends the/any updated information to the LCDproc
# server
sub update {
	foreach $project ( @projects ) {
		@_ = split( ' ', $stats{"rates-".$project} );
		$factor = 1;
		foreach( @_ ) {
			if( $_ > $factor ) {
				$factor = $_;
			}
		}
		for( $i = 0; $i < 5; $i++ ) {
			$tmp = $i + 1;
			
			# This only reared it's head when the OGR project
			# restarted and I started getting "divide by zero"
			# errors cropping up, DOH!
			if( $_[$i] > 0 ) {
				$scale = ( $_[$i] / $factor ) * 24;
				print $remote "widget_set $project $project-cg$i $tmp 4 $scale\n";
			} else {
				print $remote "widget_set $project $project-cg$i $tmp 4 0\n";
			}
		}
		print $remote "widget_set $project $project-lt 7 2 20 2 h 2 {LAST=$stats{\"rate-\".$project}}\n";
		print $remote "widget_set $project $project-at 7 3 20 3 h 2 {TOTAL=$stats{\"total-\".$project}}\n";
		print $remote "widget_set $project $project-bt 7 4 20 4 h 2 {OUT=$stats{\"pdone-\".$project}($stats{\"udone-\".$project}), IN=$stats{\"pleft-\".$project}($stats{\"uleft-\".$project})}\n";
	}
}

