#!/usr/bin/perl
#
# LCDproc client for distributed.net personal proxy 1.1
# Copyright (C) 2000  Matthew Dainty (madmatt@bits.bris.ac.uk)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: dnetc-proxy.pl,v 1.3 2000/07/15 03:08:41 matt Exp $

use IO::Socket;
use Fcntl;
use File::Tail 0.91;

# Change these as necessary for your setup, they are pretty much
# the default
my $filenameprefix = "/var/log/dnetc/proxy";
my $filenamesuffix = ".log";

# This is the list of projects you want to monitor. Currently you can
# choose from:
#
# o rc564
# o ogr
# o desII
# o csc
#
# The last two are purely for completion as they are no longer running.
# OGR will probably start again at some point <shrug>
#
# Example, to monitor rc564 and ogr, change the line below to:
#
# my @projects = ( "rc564", "ogr" );
#
# NOTE: Make sure you get the case right!
my @projects = ( "rc564", "ogr" );

my %stats;
my $filename = getfilename();

# Quickly rip through the log file and get the latest stats
if( -f $filename ) {
	open IN, $filename or die "Cannot open proxy log file\n";
	while( <IN> ) {
		carve( $_ );
	}
	close IN;
} else {
	die "Cannot locate any recent proxy log file(s)\n";
}

# We want to now tail this log file for any updates 
push( @files, File::Tail->new( name=>$filename, debug=>0 ) );

# Connect to the LCDproc server
$remote = IO::Socket::INET->new(
		Proto     => "tcp",
		PeerAddr  => "localhost",
		PeerPort  => "13666",
	)
	|| die "Cannot connect to LCDproc port\n";

# Make sure our messages get there right away
$remote->autoflush( 1 );

# Give the server a little time to realise we're here
sleep 1;

# Connect to the LCDproc server (easy-peasy)
print $remote "hello\n";
my $lcdconnect = <$remote>;

# This contains the various bits of info about the LCD screen we are
# displaying on
#
# FIXME: Maybe add code so that this will work on screens other than
# 20x4 in size. 20x2, etc.
# print $lcdconnect;

# Turn off blocking mode
fcntl( $remote, F_SETFL, O_NONBLOCK );

# Construct a new LCDproc screen for each project we want to display
print $remote "client_set name {distributed.net proxy}\n";
foreach $project ( @projects ) {

	# Add a screen
	print $remote "screen_add $project\n";
	print $remote "screen_set $project priority 128 name {Proxy for distributed.net $project}\n";

	# Add the title for the screen
	print $remote "widget_add $project $project-title title\n";
	print $remote "widget_set $project $project-title {Proxy for distributed.net $project}\n";

	# Add the various widgets
	print $remote "widget_add $project $project-ig hbar\n";
	print $remote "widget_add $project $project-og hbar\n";
	print $remote "widget_add $project $project-it scroller\n";
	print $remote "widget_add $project $project-ot scroller\n";
	print $remote "widget_add $project $project-tt scroller\n";
}

# Set the widgets up with the initial values
update();

# Loop for the rest of eternity
while( 1 ) {
	
	# Tail the log file and timeout after 2 seconds
	$nfound = File::Tail::select( undef, undef, undef, 2, @files );
	unless( $nfound ) {

		# We had a timeout, therefore we might like to check
		# to see if there is a newer log file we ought to be
		# tail'ing
		$newfilename = getfilename();
		if( ( -f $newfilename ) && ( $newfilename ne $filename ) ) {

			# There's a new file, so pop off the old log
			# file and push on the latest one
			$tmp = pop( @files );
			$tmp->CLOSE;
			push( @files, File::Tail->new( name=>$newfilename, debug=>0 ) );
			$filename = $newfilename;
		}
	}
	
	# Get any new messages from the LCDproc server, (we don't do
	# anything with 'em yet)
	while( defined( <$remote> ) ) {

		# Do something with the input, eg. trap for closing...
	}

	# If the log file has some new input to be read
	foreach( @files ) {
		if( ! $_->predict ) {
			carve( $_->read );
		}
	}

	# Update the LCDproc widgets
	update();
	sleep 1;
}

# This is here for syntactic completeness, it never runs!
close( $remote )            || die "close: $!";
exit;

# Little subroutine to split a matching log entry and dump the various
# bits of info into the hash ready for displaying elsewhere
sub carve {
	foreach( @_ ) {
		if( /,(\S+) r=([0-9]+)\/([0-9]+), d=([0-9]+)\/([0-9]+), (.*), tot=([0-9]+)/ ) {
			$stats{"leftfar-".$1} = $2;
			$stats{"leftmax-".$1} = $3;
			$stats{"donefar-".$1} = $4;
			$stats{"donemax-".$1} = $5;
			$stats{"rate-".$1} = $6;
			$stats{"total-".$1} = $7;
		}
	}
}

# This subroutine sends the/any updated information to the LCDproc
# server
sub update {
	foreach $project ( @projects ) {
		
		# Fixed a bug here! The output buffer is now included in
		# calculating the scaling factor for the bar graphs
		@_ = sort { $b <=> $a; } $stats{"leftfar-".$project}, $stats{"leftmax-".$project}, $stats{"donefar-".$project};
		$inputscale = ( $stats{"leftfar-".$project} / $_[0] ) * 50;
		$outputscale = ( $stats{"donefar-".$project} / $_[0] ) * 50;
		print $remote "widget_set $project $project-ig 1 2 $inputscale\n";
		print $remote "widget_set $project $project-og 1 3 $outputscale\n";
		print $remote "widget_set $project $project-it 12 2 20 2 h 2 {$stats{\"leftfar-\".$project}/$stats{\"leftmax-\".$project}}\n";
		print $remote "widget_set $project $project-ot 12 3 20 3 h 2 {$stats{\"donefar-\".$project}/$stats{\"donemax-\".$project}}\n";
		print $remote "widget_set $project $project-tt 1 4 20 4 h 2 {TOT=$stats{\"total-\".$project}, AVG=$stats{\"rate-\".$project}}\n";
	}
}

# This subroutine gets the latest proxy log filename
sub getfilename {

	# First try todays date for the proxy log file
	( undef, undef, undef, $mday, $mon, $year, undef, undef, undef ) = localtime time();
	$newname = sprintf "%s%d%02d%02d%s", $filenameprefix, 1900 + $year, $mon + 1, $mday, $filenamesuffix;

	# Sometimes it takes a while to roll over so drop back to
	# yesterdays date
	if( ! -f $newname ) {
		( undef, undef, undef, $mday, $mon, $year, undef, undef, undef ) = localtime( time() - ( 24 * 60 * 60 ) );
		$newname = sprintf "%s%d%02d%02d%s", $filenameprefix, 1900 + $year, $mon + 1, $mday, $filenamesuffix;
	}
	return $newname;
}
